@@ -1,13 +1,10 @@
 import logging
 import threading
-
-import six
-from six.moves.urllib.parse import parse_qs, urlparse
+from urllib.parse import parse_qs, urlparse
 
 from sqlalchemy import create_engine
 from sqlalchemy.sql import text
 from sqlalchemy.schema import MetaData
-from sqlalchemy.pool import StaticPool
 from sqlalchemy.util import safe_reraise
 from sqlalchemy.engine.reflection import Inspector
 
@@ -33,10 +30,10 @@ class Database(object):
             engine_kwargs = {}
 
         parsed_url = urlparse(url)
-        if parsed_url.scheme.lower() in 'sqlite':
-            # ref: https://github.com/pudo/dataset/issues/163
-            if 'poolclass' not in engine_kwargs:
-                engine_kwargs['poolclass'] = StaticPool
+        # if parsed_url.scheme.lower() in 'sqlite':
+        #     # ref: https://github.com/pudo/dataset/issues/163
+        #     if 'poolclass' not in engine_kwargs:
+        #         engine_kwargs['poolclass'] = StaticPool
 
         self.lock = threading.RLock()
         self.local = threading.local()
@@ -130,21 +127,37 @@ class Database(object):
         if error_type is None:
             try:
                 self.commit()
-            except:
+            except Exception:
                 with safe_reraise():
                     self.rollback()
         else:
             self.rollback()
 
+    def close(self):
+        """Close database connections. Makes this object unusable."""
+        self.engine.dispose()
+        self._tables = {}
+        self.engine = None
+
     @property
     def tables(self):
         """Get a listing of all tables that exist in the database."""
         return self.inspect.get_table_names(schema=self.schema)
 
+    @property
+    def views(self):
+        """Get a listing of all views that exist in the database."""
+        return self.inspect.get_view_names(schema=self.schema)
+
     def __contains__(self, table_name):
         """Check if the given table name exists in the database."""
         try:
-            return normalize_table_name(table_name) in self.tables
+            table_name = normalize_table_name(table_name)
+            if table_name in self.tables:
+                return True
+            if table_name in self.views:
+                return True
+            return False
         except ValueError:
             return False
 
@@ -176,7 +189,7 @@ class Database(object):
             table5 = db.create_table('population5',
                                      primary_id=False)
         """
-        assert not isinstance(primary_type, six.string_types), \
+        assert not isinstance(primary_type, str), \
             'Text-based primary_type support is dropped, use db.types.'
         table_name = normalize_table_name(table_name)
         with self.lock:
@@ -221,6 +234,10 @@ class Database(object):
         """Get a given table."""
         return self.get_table(table_name)
 
+    def _ipython_key_completions_(self):
+        """Completion for table names with IPython."""
+        return self.tables
+
     def query(self, query, *args, **kwargs):
         """Run a statement on the database directly.
 
@@ -243,9 +260,11 @@ class Database(object):
 
         The returned iterator will yield each result sequentially.
         """
-        if isinstance(query, six.string_types):
+        if isinstance(query, str):
             query = text(query)
         _step = kwargs.pop('_step', QUERY_STEP)
+        if _step is False or _step == 0:
+            _step = None
         rp = self.executable.execute(query, *args, **kwargs)
         return ResultIter(rp, row_type=self.row_type, step=_step)
 
