@@ -1,15 +1,12 @@
+# coding: utf-8
 from __future__ import unicode_literals
 
+from collections import OrderedDict
 import os
 import unittest
 from datetime import datetime
 
-try:
-    from collections import OrderedDict
-except ImportError:  # pragma: no cover
-    from ordereddict import OrderedDict  # Python < 2.7 drop-in
-
-from sqlalchemy import FLOAT, INTEGER, TEXT
+from sqlalchemy import FLOAT, TEXT, BIGINT
 from sqlalchemy.exc import IntegrityError, SQLAlchemyError, ArgumentError
 
 from dataset import connect
@@ -235,6 +232,12 @@ class TableTestCase(unittest.TestCase):
             )
         assert len(self.tbl) == len(TEST_DATA) + 1, len(self.tbl)
 
+    def test_upsert_id(self):
+        table = self.db['banana_with_id']
+        data = dict(id=10, title='I am a banana!')
+        table.upsert(data, ['id'])
+        assert len(table) == 1, len(table)
+
     def test_update_while_iter(self):
         for row in self.tbl:
             row['foo'] = 'bar'
@@ -250,6 +253,22 @@ class TableTestCase(unittest.TestCase):
                 'qux.bar': 'Huhu'
             })
 
+    def test_cased_column_names(self):
+        tbl = self.db['cased_column_names']
+        tbl.insert({
+            'place': 'Berlin',
+        })
+        tbl.insert({
+            'Place': 'Berlin',
+        })
+        tbl.insert({
+            'PLACE ': 'Berlin',
+        })
+        assert len(tbl.columns) == 2, tbl.columns
+        assert len(list(tbl.find(Place='Berlin'))) == 3
+        assert len(list(tbl.find(place='Berlin'))) == 3
+        assert len(list(tbl.find(PLACE='Berlin'))) == 3
+
     def test_invalid_column_names(self):
         tbl = self.db['weather']
         with self.assertRaises(ValueError):
@@ -300,8 +319,8 @@ class TableTestCase(unittest.TestCase):
 
     def test_count(self):
         assert len(self.tbl) == 6, len(self.tbl)
-        l = self.tbl.count(place=TEST_CITY_1)
-        assert l == 3, l
+        length = self.tbl.count(place=TEST_CITY_1)
+        assert length == 3, length
 
     def test_find(self):
         ds = list(self.tbl.find(place=TEST_CITY_1))
@@ -321,6 +340,26 @@ class TableTestCase(unittest.TestCase):
         ds = list(self.tbl.find(self.tbl.table.columns.temperature > 4))
         assert len(ds) == 3, ds
 
+    def test_find_dsl(self):
+        ds = list(self.tbl.find(place={'like': '%lw%'}))
+        assert len(ds) == 3, ds
+        ds = list(self.tbl.find(temperature={'>': 5}))
+        assert len(ds) == 2, ds
+        ds = list(self.tbl.find(temperature={'>=': 5}))
+        assert len(ds) == 3, ds
+        ds = list(self.tbl.find(temperature={'<': 0}))
+        assert len(ds) == 1, ds
+        ds = list(self.tbl.find(temperature={'<=': 0}))
+        assert len(ds) == 2, ds
+        ds = list(self.tbl.find(temperature={'!=': -1}))
+        assert len(ds) == 5, ds
+        ds = list(self.tbl.find(temperature={'between': [5, 8]}))
+        assert len(ds) == 3, ds
+        ds = list(self.tbl.find(place={'=': 'G€lway'}))
+        assert len(ds) == 3, ds
+        ds = list(self.tbl.find(place={'ilike': '%LwAy'}))
+        assert len(ds) == 3, ds
+
     def test_offset(self):
         ds = list(self.tbl.find(place=TEST_CITY_1, _offset=1))
         assert len(ds) == 2, ds
@@ -344,11 +383,40 @@ class TableTestCase(unittest.TestCase):
             self.tbl.table.columns.date >= datetime(2011, 1, 2, 0, 0)))
         assert len(x) == 4, x
 
+        x = list(self.tbl.distinct('temperature', place='B€rkeley'))
+        assert len(x) == 3, x
+        x = list(self.tbl.distinct('temperature',
+                                   place=['B€rkeley', 'G€lway']))
+        assert len(x) == 6, x
+
     def test_insert_many(self):
         data = TEST_DATA * 100
         self.tbl.insert_many(data, chunk_size=13)
         assert len(self.tbl) == len(data) + 6
 
+    def test_update_many(self):
+        tbl = self.db['update_many_test']
+        tbl.insert_many([
+            dict(temp=10), dict(temp=20), dict(temp=30)
+        ])
+        tbl.update_many(
+            [dict(id=1, temp=50), dict(id=3, temp=50)], 'id'
+        )
+
+        # Ensure data has been updated.
+        assert tbl.find_one(id=1)['temp'] == tbl.find_one(id=3)['temp']
+
+    def test_upsert_many(self):
+        # Also tests updating on records with different attributes
+        tbl = self.db['upsert_many_test']
+
+        W = 100
+        tbl.upsert_many([dict(age=10), dict(weight=W)], 'id')
+        assert tbl.find_one(id=1)['age'] == 10
+
+        tbl.upsert_many([dict(id=1, age=70), dict(id=2, weight=W / 2)], 'id')
+        assert tbl.find_one(id=2)['weight'] == W / 2
+
     def test_drop_operations(self):
         assert self.tbl._table is not None, \
             'table shouldn\'t be dropped yet'
@@ -416,12 +484,20 @@ class TableTestCase(unittest.TestCase):
             tbl.table.c['bar'].type
         tbl.create_column_by_example('bar', 1)
         assert 'bar' in tbl.table.c, tbl.table.c
-        assert isinstance(tbl.table.c['bar'].type, INTEGER), \
+        assert isinstance(tbl.table.c['bar'].type, BIGINT), \
             tbl.table.c['bar'].type
         tbl.create_column_by_example('pippo', 'test')
         assert 'pippo' in tbl.table.c, tbl.table.c
         assert isinstance(tbl.table.c['pippo'].type, TEXT), \
             tbl.table.c['pippo'].type
+        tbl.create_column_by_example('bigbar', 11111111111)
+        assert 'bigbar' in tbl.table.c, tbl.table.c
+        assert isinstance(tbl.table.c['bigbar'].type, BIGINT), \
+            tbl.table.c['bigbar'].type
+        tbl.create_column_by_example('littlebar', -11111111111)
+        assert 'littlebar' in tbl.table.c, tbl.table.c
+        assert isinstance(tbl.table.c['littlebar'].type, BIGINT), \
+            tbl.table.c['littlebar'].type
 
     def test_key_order(self):
         res = self.db.query('SELECT temperature, place FROM weather LIMIT 1')
@@ -430,9 +506,8 @@ class TableTestCase(unittest.TestCase):
         assert keys[1] == 'place'
 
     def test_empty_query(self):
-        m = self.tbl.find(place='not in data')
-        l = list(m)  # exhaust iterator
-        assert len(l) == 0
+        empty = list(self.tbl.find(place='not in data'))
+        assert len(empty) == 0, empty
 
 
 class Constructor(dict):
